'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _config = require('../../config/config');

var _config2 = _interopRequireDefault(_config);

var _logger = require('../../logger');

var _logger2 = _interopRequireDefault(_logger);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Sequelize = require('sequelize');
var Promise = require('bluebird');


// Connection url
var url = _config2.default.get('SEQUELIZE_CONNECTION_STRING') || 'mongodb://localhost:27017/test2';
var db = void 0;
var sequelize = new Sequelize(url, {

	// disable logging; default: console.log
	logging: false

});

function roughSizeOfObject(object, thresh) {
	function literalSize(value) {
		var theType = typeof value === 'undefined' ? 'undefined' : (0, _typeof3.default)(value);
		if (theType === 'boolean') {
			return 4;
		} else if (theType === 'string') {
			return value.length * 2;
		} else if (theType === 'number') {
			return 8;
		}
	}
	var objectList = [];
	var stack = [object];
	var bytes = 0;
	while (stack.length) {
		var value = stack.pop();
		if ((typeof value === 'undefined' ? 'undefined' : (0, _typeof3.default)(value)) === 'object' && objectList.indexOf(value) === -1) {
			objectList.push(value);
			for (var i in value) {
				// key
				bytes = bytes + i.toString().length;
				stack.push(value[i]);
			}
		} else {
			bytes += literalSize(value);
		}
		if (bytes >= thresh) {
			return bytes;
		}
	}
	return bytes;
}

var Entry = sequelize.define('blob', {
	createdAt: Sequelize.DATE,
	key: {
		type: Sequelize.STRING,
		primaryKey: true
	},
	data: {
		type: Sequelize.TEXT
	}
}, {
	indexes: [{
		fields: ['key']
	}],
	createdAt: 'createdAt'
});

var keyNum = 0;
// TODO: lookup between id and collection name
var plugin = {
	init: function () {
		var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
			return _regenerator2.default.wrap(function _callee$(_context) {
				while (1) {
					switch (_context.prev = _context.next) {
						case 0:
							try {
								sequelize.authenticate().then(function () {
									_logger2.default.info('Connection has been established successfully.');
									return Entry.sync({});
								}).then(function () {
									_logger2.default.info('Table created/updated');
									// Table created
								}).catch(function (err) {
									_logger2.default.error('Unable to connect to the database:', err);
									process.exit(-1);
								});
							} catch (e) {
								process.exit(-1);
							}

						case 1:
						case 'end':
							return _context.stop();
					}
				}
			}, _callee, this);
		}));

		function init() {
			return _ref.apply(this, arguments);
		}

		return init;
	}(),
	getData: function () {
		var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(_ref3) {
			var key = _ref3.key;
			return _regenerator2.default.wrap(function _callee2$(_context2) {
				while (1) {
					switch (_context2.prev = _context2.next) {
						case 0:
							return _context2.abrupt('return', Entry.findAll({
								where: { key: key }
							}).then(function (entries) {
								// logger.debug('data in entries',id,entries,entries.length);
								var result = entries.map(function (e) {
									return JSON.parse(e.data);
								});
								return result.length ? result[0] : null;
							}));

						case 1:
						case 'end':
							return _context2.stop();
					}
				}
			}, _callee2, this);
		}));

		function getData(_x) {
			return _ref2.apply(this, arguments);
		}

		return getData;
	}(),
	setData: function () {
		var _ref4 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(_ref5) {
			var entry = _ref5.entry;
			var thresh, key;
			return _regenerator2.default.wrap(function _callee3$(_context3) {
				while (1) {
					switch (_context3.prev = _context3.next) {
						case 0:
							thresh = process.env.BLOB_MIN_SIZE || 100;

							if (!(roughSizeOfObject(entry, thresh) >= thresh)) {
								_context3.next = 8;
								break;
							}

							key = 'key' + keyNum++;
							//entries[key] = entry;

							_context3.next = 5;
							return Entry.upsert({
								key: key,
								data: (0, _stringify2.default)(entry)
							});

						case 5:
							return _context3.abrupt('return', { _blobKey: key });

						case 8:
							return _context3.abrupt('return', entry);

						case 9:
						case 'end':
							return _context3.stop();
					}
				}
			}, _callee3, this);
		}));

		function setData(_x2) {
			return _ref4.apply(this, arguments);
		}

		return setData;
	}()
};

exports.default = plugin;