'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _config = require('../../config/config');

var _config2 = _interopRequireDefault(_config);

var _logger = require('../../logger');

var _logger2 = _interopRequireDefault(_logger);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Sequelize = require('sequelize');
var Promise = require('bluebird');


// Connection url
var url = _config2.default.get('SEQUELIZE_CONNECTION_STRING') || 'mongodb://localhost:27017/test2';
var db = void 0;
var sequelize = new Sequelize(url, {

	// disable logging; default: console.log
	logging: false

});

var Entry = sequelize.define('entry', {
	createdAt: Sequelize.DATE,
	journal: Sequelize.STRING,
	id: {
		type: Sequelize.INTEGER,
		primaryKey: true,
		autoIncrement: true
	},
	data: {
		type: Sequelize.TEXT
	}
}, {
	indexes: [
	// Create a unique index on poem
	{
		fields: ['journal']
	}],
	createdAt: 'createdAt'
});

// TODO: lookup between id and collection name
var plugin = {
	init: function () {
		var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
			return _regenerator2.default.wrap(function _callee$(_context) {
				while (1) {
					switch (_context.prev = _context.next) {
						case 0:
							try {
								sequelize.authenticate().then(function () {
									_logger2.default.info('Connection has been established successfully.');
									return Entry.sync({});
								}).then(function () {
									_logger2.default.info('Table created/updated');
									// Table created
								}).catch(function (err) {
									_logger2.default.error('Unable to connect to the database:', err);
									process.exit(-1);
								});
							} catch (e) {
								process.exit(-1);
							}

						case 1:
						case 'end':
							return _context.stop();
					}
				}
			}, _callee, this);
		}));

		function init() {
			return _ref.apply(this, arguments);
		}

		return init;
	}(),
	getEntries: function () {
		var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(_ref3) {
			var id = _ref3.id;
			return _regenerator2.default.wrap(function _callee2$(_context2) {
				while (1) {
					switch (_context2.prev = _context2.next) {
						case 0:
							return _context2.abrupt('return', Entry.findAll({ where: { journal: id }, order: [['id']] }).then(function (entries) {
								// logger.debug('data in entries',id,entries,entries.length);
								return entries.map(function (e) {
									return JSON.parse(e.data);
								});
							}).then(function (entries) {
								// logger.debug('data in entries after map',id,entries,entries.length);
								return entries;
							}));

						case 1:
						case 'end':
							return _context2.stop();
					}
				}
			}, _callee2, this);
		}));

		function getEntries(_x) {
			return _ref2.apply(this, arguments);
		}

		return getEntries;
	}(),
	getJournals: function () {
		var _ref4 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(_ref5) {
			var debug = _ref5.debug;
			return _regenerator2.default.wrap(function _callee3$(_context3) {
				while (1) {
					switch (_context3.prev = _context3.next) {
						case 0:
							_context3.next = 2;
							return Entry.findAll({ attributes: ['journal'], group: ["journal"] }).then(function (entries) {
								return entries.map(function (e) {
									return e.journal;
								});
							});

						case 2:
							return _context3.abrupt('return', _context3.sent);

						case 3:
						case 'end':
							return _context3.stop();
					}
				}
			}, _callee3, this);
		}));

		function getJournals(_x2) {
			return _ref4.apply(this, arguments);
		}

		return getJournals;
	}(),
	clear: function () {
		var _ref6 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4(_ref7) {
			var id = _ref7.id;
			return _regenerator2.default.wrap(function _callee4$(_context4) {
				while (1) {
					switch (_context4.prev = _context4.next) {
						case 0:
							_context4.next = 2;
							return Entry.destroy({
								where: {
									journal: id
								}
							});

						case 2:
							return _context4.abrupt('return', _context4.sent);

						case 3:
						case 'end':
							return _context4.stop();
					}
				}
			}, _callee4, this);
		}));

		function clear(_x3) {
			return _ref6.apply(this, arguments);
		}

		return clear;
	}(),
	append: function () {
		var _ref8 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee5(_ref9) {
			var entry = _ref9.entry,
			    id = _ref9.id;
			return _regenerator2.default.wrap(function _callee5$(_context5) {
				while (1) {
					switch (_context5.prev = _context5.next) {
						case 0:
							_context5.next = 2;
							return Entry.create({
								journal: id,
								data: (0, _stringify2.default)(entry)
							});

						case 2:
							return _context5.abrupt('return', _context5.sent);

						case 3:
						case 'end':
							return _context5.stop();
					}
				}
			}, _callee5, this);
		}));

		function append(_x4) {
			return _ref8.apply(this, arguments);
		}

		return append;
	}()
};

exports.default = plugin;